/*
 * BOX OFFICES for iPhone/iPad
 *
 * The Original Game is Designed by Entatonic.net
 * Copyright (c) 2010 Entatonic.net
 * http://entatonic.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */



#import "Launcher.h"
#import "iPhone_iPad_Selector.h"

@implementation Launcher

@synthesize listOnlauncher;

-(id) initWithcpSpace: (cpSpace *) space{
	self = [super init];
	if(self != nil){
		listOnlauncher = [[NSMutableArray alloc] init];	// Launcher Initialize
		m_space = space;
		[self resetLauncher];
		
		// Display Size
		CGSize s = [[CCDirector sharedDirector] winSize];
		display_width = s.width;
		display_height = s.height;
		display_width_center = display_width / 2;
		display_height_launcher_y = display_height / 24;
	}
	return self;
}

-(void) resetLauncher{
	for(int i=[listOnlauncher count] -1; i >= 0 ;i--){
		[listOnlauncher removeObjectAtIndex:i];
	}
	
	m_move_offset = 0;
	moment_counter = -1;
	moment_vx = 0;
	boxExpand_interval_counter = BOXExpand_interval;
	init_launchingBox_size = LAUNCHING_BOX_SIZE;
	launchingBox_size = init_launchingBox_size;								// %
	expiredLauncher_MoveLauncherToLeft = FALSE;
	LauncherExpandSpeedRate = 100;
}

-(void) moveLauncherLeft{
	expiredLauncher_MoveLauncherToLeft = FALSE;
	
	m_move_offset = LAUNCHER_ANIMATION_DIRECTION;
	moment_counter = MOMENT_COUNTER_FORANGLE;
	moment_vx = -1;	// left

	[[MyOpenAL sharedMyOpenAL] playSoundWithKey:@"Factory"];
}

-(void) moveLauncherListLeft{
	if([listOnlauncher count] > 0)
		[listOnlauncher removeObjectAtIndex:0];
}

-(void) addOboxObject:(boxObject*) box{
	[listOnlauncher addObject:box];
}


-(void) setLaunchingBoxScale{
	[[listOnlauncher objectAtIndex:0] setBoxScale:launchingBox_size];
}

-(BOOL) launchBox_vx:(float)launcher_vx vy:(float)launcher_vy{
	if([listOnlauncher count] > 0){
		if([[listOnlauncher objectAtIndex:0] inField] == TRUE)	// If box on launcher is already in field, skip the launcher
			return FALSE;	// Do nothing
		
		[[listOnlauncher objectAtIndex:0] createBoxInstance:launcher_vx vy:launcher_vy];	// flick into field
		launchingBox_size = init_launchingBox_size;
		return TRUE;		// Launched
	}
	return FALSE;
}

-(BOOL) expiredLauncher_MoveLauncherToLeft{
	return expiredLauncher_MoveLauncherToLeft;
}

// return to be Invalid object
-(boxObject*) stepAnimation:(StageManager *) stageManager{
	boxObject* invalidObject = 0;		// to response to caller which box is invalid
	
	// Count the interval of Box Expand on launching posion
	if(boxExpand_interval_counter > 0){
		boxExpand_interval_counter--;
		if(boxExpand_interval_counter == 0){
			if(launchingBox_size >= AUTO_RELEASE_BOX_SIDE){						// AUTO LAUNCH
				[self launchBox_vx:0 vy:300 * iPhone_iPad];
			}else{
					launchingBox_size = launchingBox_size + 10;
			}
			boxExpand_interval_counter = BOXExpand_interval * LauncherExpandSpeedRate / 100;
			expiredLauncher_MoveLauncherToLeft = TRUE;
		}
	}
	
	// Slide before or after moment Animation
	float angle;

	if(moment_counter > 0){
		moment_counter--;
		if(moment_counter >= 0){
			angle = (float) (MOMENT_COUNTER_FORANGLE / 2) - abs(MOMENT_COUNTER_FORANGLE / 2 - moment_counter);
			if(angle != 0)
				angle = -1 * moment_vx * (float) 2.0f * M_PI / 360.0f / angle * 20.0f;
			for(int i=1; i < [listOnlauncher count];i++){
				[[listOnlauncher objectAtIndex:i] setAngle:angle];
			}		
			return invalidObject;
		}
	}

	// Slide Animation
	int withLaunchingBox = 1;	// Basically the slide animation includes launching box
	int launcher_offset = 0;
	
	if(m_move_offset != 0){
		if(m_move_offset < 0){
			m_move_offset++;
			if(m_move_offset==0) {	// Slide End
				boxObject* boxCheck = [listOnlauncher objectAtIndex:0];
				if([boxCheck inField] == FALSE){
					invalidObject = boxCheck;		// it's not in field(=on launcher), set the box invalid to delete in field step animation
					[[MyOpenAL sharedMyOpenAL] playSoundWithKey:@"Factory2"];
				}
				[self moveLauncherListLeft];		// refresh the list	
				[self setLaunchingBoxScale];
				withLaunchingBox = 0;				// redraw all
				moment_counter = MOMENT_COUNTER_FORANGLE;
				moment_vx = 1;
			}	//left
		}

		if(m_move_offset == 0){
			launcher_offset = 0;	// Sliding Completed and the list is refreshed
		}else {
			launcher_offset = LAUNCHER_ANIMATION_DIRECTION - m_move_offset;
		}
	
		for(int i=withLaunchingBox; i < [listOnlauncher count];i++){
			[[listOnlauncher objectAtIndex:i]
			 setXY:[self getXFromIndex:i] + launcher_offset  * iPhone_iPad y:[self getYFromIndex:i]];
		}
	}
	
	return invalidObject;
}

-(boxObject *) getboxObjectIndex:(int) index{
	return [listOnlauncher objectAtIndex:index];
}

-(void) setLauncherExpandSpeedRate:(int) rate{
	if(LauncherExpandSpeedRate >=0 && LauncherExpandSpeedRate <= 100){
		LauncherExpandSpeedRate = rate;
	}else{
		LauncherExpandSpeedRate = 100;
	}
}

-(float) getXFromIndex:(int) i{
	return display_width_center + i * LAUNCHER_INTERVAL * iPhone_iPad;
}

-(float) getYFromIndex:(int) i{
	if(i == 0){
		return display_height_launcher_y + 25 * iPhone_iPad;
	}else if(i ==1){
		if(m_move_offset == 0)
			return display_height_launcher_y;
		else
			return display_height_launcher_y + 25  * iPhone_iPad + m_move_offset * iPhone_iPad;
	}else{
		return display_height_launcher_y;
	}
}

-(int) getNumOfObjects{
	if([listOnlauncher count] > 0){
		if([[listOnlauncher objectAtIndex:0] inField] == TRUE)
			return [listOnlauncher count] - 1;	// the box is already in field
		else
			return [listOnlauncher count];
	}else{
		return 0;
	}
}


-(void) dealloc
{
	[super dealloc];
}

@end
