/*
 * BOX OFFICES for iPhone/iPad
 *
 * The Original Game is Designed by Entatonic.net
 * Copyright (c) 2010 Entatonic.net
 * http://entatonic.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */



#import "ScoreManager.h"


@implementation ScoreManager

@synthesize addingScore_x, addingScore_y, addingScore, addingScore_displaying, addingScore_displaying_counter;

-(id) init {
	self = [super init];
	if(self != nil){
		self.addingScore_x	=					[[NSMutableArray alloc] init];
		self.addingScore_y	=					[[NSMutableArray alloc] init];
		self.addingScore	=					[[NSMutableArray alloc] init];
		self.addingScore_displaying	=			[[NSMutableArray alloc] init];
		self.addingScore_displaying_counter	=	[[NSMutableArray alloc] init];
		[self resetScore];
	}
	return self;
}

-(void) addScore:(int) PlusScore x:(int) score_display_x y:(int) score_display_y{
	score = score + PlusScore;
	[self addAddingScore:PlusScore x:score_display_x y:score_display_y];
}

-(void) resetScore{
	score = 0;
	displayingScore = 0;
	[self resetAddingScore];
}

-(void) resetAddingScore{
	for(int i=[self.addingScore count] -1; i >= 0 ;i--){
		[self removeAddingScore:i];
	}
}

// sync displaying score with real score
-(void) syncScore{
	displayingScore = score;
}

-(int) getScore{
	return score;
}

-(int) getDisplayingScore{
	return displayingScore;
}

-(void) addAddingScore:(int)add_score x:(int)x y:(int)y{
	[self.addingScore_x				addObject:[NSNumber numberWithInteger:x]];	
	[self.addingScore_y				addObject:[NSNumber numberWithInteger:y]];	
	[self.addingScore				addObject:[NSNumber numberWithInteger:add_score]];	
	[self.addingScore_displaying	addObject:[NSNumber numberWithInteger:0]];
	[self.addingScore_displaying_counter	addObject:[NSNumber numberWithInteger:ADDINGSCORE_DISPLAYING_COUNTER_MAX]];	
}

-(void) removeAddingScore:(int) index{
	[self.addingScore_x				removeObjectAtIndex:index];	
	[self.addingScore_y				removeObjectAtIndex:index];	
	[self.addingScore				removeObjectAtIndex:index];	
	[self.addingScore_displaying	removeObjectAtIndex:index];
	[self.addingScore_displaying_counter	removeObjectAtIndex:index];	
}

-(int) getNumOfAddingScore{
	return [self.addingScore count];
}

-(int) getAddingScore_x:(int) index{
	return	[[self.addingScore_x			objectAtIndex:index] intValue];
}

-(int) getAddingScore_y:(int) index{
	return	[[self.addingScore_y			objectAtIndex:index] intValue];
}

-(int) getAddingScore_addingScore_displaying:(int) index{
	return	[[self.addingScore_displaying	objectAtIndex:index] intValue];
}

-(int) getAddingScore_addingScore_displaying_counter:(int) index{
	return	[[self.addingScore_displaying_counter	objectAtIndex:index] intValue];
}

-(void) stepAnimation{
	if (score > displayingScore){
		if(score > displayingScore + 1000){
			displayingScore += 100;
		} else {
			if(score > displayingScore + 100){
				displayingScore += 50;
			} else {
				if(score > displayingScore + 10)
					displayingScore += 5;
				else
					displayingScore++;
			}
		}
	}
	
	int score_tmp;
	int displaying_score_tmp;
	for(int i=[self.addingScore count]-1; i >= 0 ;i--){
		score_tmp =				[[self.addingScore objectAtIndex:i] intValue];
		displaying_score_tmp =	[[self.addingScore_displaying objectAtIndex:i] intValue];
		if (score_tmp > displaying_score_tmp){
			if(score_tmp > displaying_score_tmp + 1000){
				displaying_score_tmp += 100;
			}else{
				if(score_tmp > displaying_score_tmp + 100){
					displaying_score_tmp += 50;
				}else{
					if(score_tmp > displaying_score_tmp + 10){
						displaying_score_tmp += 5;
					}else{
						displaying_score_tmp++;
					}
				}
			}		
			[self.addingScore replaceObjectAtIndex:i withObject:[NSNumber numberWithInteger:score_tmp]];
			[self.addingScore_displaying replaceObjectAtIndex:i withObject:[NSNumber numberWithInteger:displaying_score_tmp]];

		}else{		// fade out animation
			
			[self.addingScore_displaying replaceObjectAtIndex:i withObject:[NSNumber numberWithInteger:score_tmp]];
			int counter =	[[self.addingScore_displaying_counter objectAtIndex:i] intValue];
			counter--;
			if(counter > 0){
				[self.addingScore_displaying_counter replaceObjectAtIndex:i withObject:[NSNumber numberWithInteger:counter]];
			}else{
				[self removeAddingScore:i];
			}
		}
	}
}

@end
