/*
 * BOX OFFICES for iPhone/iPad
 *
 * The Original Game is Designed by Entatonic.net
 * Copyright (c) 2010 Entatonic.net
 * http://entatonic.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */



#import "boxObject.h"


@implementation boxObject

-(id) initWithX: (float)box_x y:(float)box_y
		boxtype:(int)box_type
		  layer:(int)box_layer  scale:(int)scale
			cpSpace:(cpSpace *) space
			imagePath:(NSString *) imagePath
			spriteOwnerCocosLayer:(CCLayer *) spriteOwnerCocosLayer
			displayingscale:(int) displayingscale
{
	self = [super init];
	if(self != nil){
		m_boxType = box_type;
		m_layer = box_layer;
		m_tag = m_layer;									// m_layer equals to m_tag
		m_spriteOwnerCocosLayer = spriteOwnerCocosLayer;

		m_space = space;
		m_scale = scale;
		m_displaying_scale = displayingscale;
		m_space_scale = scale;
		m_box_x = box_x;
		m_box_y = box_y;
		m_box_vx =0.0f;
		m_box_vy =0.0f;

		shrink_counter = SHRINK_COUNTER_INIT_VALUE;
		expand_redline_warning_counter = EXPAND_REDLINE_COUNTER_INIT_VALUE;
	
		m_imagePath = [[NSString alloc] initWithFormat:@"%@", imagePath];
		createdInstance = FALSE;	// FALSE: sprite only , TRUE: sprite with instance(Space)
		inField = FALSE;
		
		[self createSprite:imagePath x:box_x y:box_y];
	}
	return self;
}

-(void) createSprite: (NSString *) imagePath
				   x:(float) box_x y:(float) box_y
{
	m_box_x = box_x;
	m_box_y = box_y;
	m_sprite_image = [CCSprite spriteWithFile:imagePath];
	m_sprite_image.scale = (float) m_displaying_scale / 100;

	[m_sprite_image setPosition:cpv(box_x, box_y)];
	[m_spriteOwnerCocosLayer addChild:m_sprite_image z:m_layer tag:m_tag];	
}

-(void) createBoxInstance:(float)box_vx vy:(float)box_vy{
	m_box_vx = box_vx;
	m_box_vy = box_vy;	
	
	if(createdInstance == FALSE){
		m_cpBody = cpBodyNew(200.0f, INFINITY);
		m_cpBody->p = cpv(m_box_x,m_box_y);
		m_cpBody->v = cpv(box_vx, box_vy);
	
		cpSpaceAddBody(m_space, m_cpBody);
	
		cpVect verts[8];

		int scale46 = (float) m_scale * 0.46  * iPhone_iPad;
		int scale50 = (float) m_scale * 0.5    * iPhone_iPad;

		verts[0] = cpv(-scale46, -scale46);	// Init Only, can't update after created
		verts[1] = cpv(-scale50,  0);
		verts[2] = cpv(-scale46,  scale46);
		verts[3] = cpv(       0,  scale50);
		verts[4] = cpv( scale46,  scale46);
		verts[5] = cpv( scale50,  0);
		verts[6] = cpv( scale46, -scale46);
		verts[7] = cpv(       0, -scale50);
		
		m_sprite_imageShape = cpPolyShapeNew(m_cpBody, 8, verts, cpvzero);
		m_sprite_imageShape->e = 1.0f; m_sprite_imageShape->u = 0.0f;
	
		CCNode *sprite_node = [m_spriteOwnerCocosLayer getChildByTag:m_tag];
		sprite_node.scale = (float) m_displaying_scale / 100;//    / 2 * iPhone_iPad;				// Scale Control
		m_sprite_imageShape->data = sprite_node;		// sprite
		m_sprite_imageShape->collision_type = 0;	// in field, collision definition type 0 is the collision target
		cpSpaceAddShape(m_space, m_sprite_imageShape);

		createdInstance = TRUE;
		inField = TRUE;
	}
}

-(void) stepAnimation{
	// EXPAND RED LINE COUNT
	expand_redline_warning_counter--;
	if(expand_redline_warning_counter <= 0){
		expand_redline_warning_counter = EXPAND_REDLINE_COUNTER_INIT_VALUE;
	}
	
	// SHRINK COUNT
	if(createdInstance == TRUE){
		if([self isChangingSize] == FALSE && m_scale > 30){
			shrink_counter--;
			if(shrink_counter <= 0){
				[self setBoxScale:m_scale - 10];
			}
		}
	}

	// SCALE ANIMATION
	if(m_displaying_scale == m_scale) return;

	int diff_scale = m_displaying_scale - m_scale;
	if(diff_scale < 0) diff_scale = diff_scale * -1;
	
	if(diff_scale < 5){
		m_displaying_scale = m_scale;
	}

	if(m_displaying_scale > m_scale){
		if( m_displaying_scale - m_scale < 30)
			m_displaying_scale = m_displaying_scale - 1;
		else
			m_displaying_scale = m_displaying_scale - 5;
	}else
	if(m_displaying_scale < m_scale){
		if( m_scale - m_displaying_scale < 30)
			m_displaying_scale = m_displaying_scale + 1;
		else
			m_displaying_scale = m_displaying_scale + 5;
	}
	CCNode *sprite = [m_spriteOwnerCocosLayer getChildByTag:m_tag];
	sprite.scale = (float) m_displaying_scale / 100;
}

-(float) getX{
	if(createdInstance == FALSE){	// sprite
		return m_box_x;
	}else{							// cocos2d space
		return m_cpBody->p.x;
	}
}

-(float) getY{
	if(createdInstance == FALSE){
		return m_box_y;
	}else{
		return m_cpBody->p.y;
	}
}

-(float) getVX{
	if(createdInstance == FALSE){
		return m_box_vx;
	}else{
		return m_cpBody->v.x;

	}
}

-(float) getVY{
	if(createdInstance == FALSE){
		return m_box_vy;
	}else{
		return m_cpBody->v.y;
	}
}

-(float) getAngle{				// NOT USED
	if(createdInstance == FALSE){
		return m_scale;			// ************** NG not angle, and not used
	}else{
		return m_cpBody->a;
	}
}

-(void) setXY:(float)box_x y:(float) box_y{

	if(createdInstance == FALSE){
		m_box_x = box_x;
		m_box_y = box_y;
		CCNode *sprite = [m_spriteOwnerCocosLayer getChildByTag:m_layer];
		[sprite setPosition:cpv(box_x, box_y)];
	}else{
		m_cpBody->p.x = box_x;
		m_cpBody->p.y = box_y;
	}
}

-(void) setVXVY:(float)box_vx vy:(float) box_vy{
	if(createdInstance == FALSE){
	}else{
		m_cpBody->v.x = box_vx;
		m_cpBody->v.y = box_vy;
	}
}

-(void) setBoxScale:(int) scale{
	m_scale = scale;
	shrink_counter = SHRINK_COUNTER_INIT_VALUE;
}

-(void) setDisplayingScale:(int)scale{
	m_displaying_scale = scale;
	CCNode *sprite = [m_spriteOwnerCocosLayer getChildByTag:m_tag];
	sprite.scale = (float) m_displaying_scale / 100;
}

-(void) setAngle:(float) angle{
	if(createdInstance == FALSE){
		CCNode *sprite = [m_spriteOwnerCocosLayer getChildByTag:m_tag];
		sprite.rotation = angle * 50;
	}else{
		m_cpBody->a = angle;
	}
}

-(int) getScale{
	return m_scale;
}

-(int) getSpaceScale{
	return m_space_scale;
}

-(BOOL) isChangingSize{
	if(m_scale != m_displaying_scale)
		return TRUE;
	else
		return FALSE;

}

-(BOOL) createdInstance{
	return createdInstance;
}

-(BOOL) inField{
	return inField;
}

-(int) getDisplayingScale{
	return m_displaying_scale;
}

-(int) getLayer{
	return m_layer;
}

-(int) getBoxType{
	return m_boxType;
}

-(NSString*) getImagePath{
	return m_imagePath;
}

-(cpShape *) getShape{
	return m_sprite_imageShape;
}

-(CCLayer *) getspriteOwnerCocosLayer{
	return m_spriteOwnerCocosLayer;
}


-(BOOL) isREDLINE_Tutorial{
	if(m_scale > EXPLODE_SCALE - 70){
		return TRUE;
	}
	return FALSE;
}

-(BOOL) isREDLINE{
	if(m_scale > EXPLODE_SCALE - 40){
		if(expand_redline_warning_counter >
		   EXPAND_REDLINE_COUNTER_INIT_VALUE / 2){
			return TRUE;
		}
	}
	return FALSE;
}

-(void) releaseShapes{
	if(createdInstance == TRUE){
		m_box_x = m_cpBody->p.x;
		m_box_y = m_cpBody->p.y;
		m_box_vx = m_cpBody->v.x;
		m_box_vy = m_cpBody->v.y;
		createdInstance = FALSE;
	}
	
	if(m_sprite_image!=nil){
		[m_spriteOwnerCocosLayer removeChild:m_sprite_image cleanup:YES];
	}
	
	if(m_sprite_imageShape!=nil){
		cpSpaceRemoveShape(m_space, m_sprite_imageShape);
		cpShapeFree(m_sprite_imageShape);
	}
	
	if(m_cpBody != nil){
		cpSpaceRemoveBody(m_space, m_cpBody);
		cpBodyFree(m_cpBody);
	}	
}

-(void) dealloc
{
	[self releaseShapes];
	[m_imagePath release];
	[super dealloc];
}

@synthesize m_layer, m_tag, m_scale, m_displaying_scale, m_space_scale, m_spriteOwnerCocosLayer;
@synthesize m_space, m_sprite_imageShape, m_sprite_image;

@end
